export const root: string
export const tooltip: string
export const left: string
export const tooltip_arrow: string
export const right: string
export const above: string
export const below: string
export const tooltip_row_label: string
export const tooltip_row_value: string
export const tooltip_color_block: string
export default ``
